import { MessagePort } from 'worker_threads';
import { Awaitable } from '@antfu/utils';
import { TransformResult, ViteDevServer } from 'vite';
import { SnapshotStateOptions } from 'jest-snapshot/build/State';

declare class StateManager {
    filesMap: Record<string, File>;
    idMap: Record<string, Task>;
    taskFileMap: WeakMap<Task, File>;
    getFiles(keys?: string[]): File[];
    collectFiles(files: File[]): void;
    updateId(task: Task): void;
    updateTasks(packs: TaskResultPack[]): void;
}

interface SnapshotResult {
    filepath: string;
    added: number;
    fileDeleted: boolean;
    matched: number;
    unchecked: number;
    uncheckedKeys: Array<string>;
    unmatched: number;
    updated: number;
}
interface UncheckedSnapshot {
    filePath: string;
    keys: Array<string>;
}
interface SnapshotSummary {
    added: number;
    didUpdate: boolean;
    failure: boolean;
    filesAdded: number;
    filesRemoved: number;
    filesRemovedList: Array<string>;
    filesUnmatched: number;
    filesUpdated: number;
    matched: number;
    total: number;
    unchecked: number;
    uncheckedKeysByFile: Array<UncheckedSnapshot>;
    unmatched: number;
    updated: number;
}

declare class SnapshotManager {
    config: ResolvedConfig;
    summary: SnapshotSummary;
    constructor(config: ResolvedConfig);
    clear(): void;
    add(result: SnapshotResult): void;
}

interface UserOptions {
    /**
     * Include globs for test files
     *
     * @default ['**\/*.test.ts']
     */
    includes?: string[];
    /**
     * Exclude globs for test files
     * @default ['**\/node_modules\/**']
     */
    excludes?: string[];
    /**
     * Handling for dependencies inlining or externalizing
     */
    deps?: {
        /**
         * Externalize means that Vite will bypass the package to native Node.
         *
         * Externaled dependencies will not be applied Vite's transformers and resolvers.
         * And does not support HMR on reload.
         *
         * Typically, packages under `node_modules` are externalized.
         */
        external?: (string | RegExp)[];
        /**
         * Vite will process inlined modules.
         *
         * This could be helpful to handle packages that ship `.js` in ESM format (that Node can't handle).
         */
        inline?: (string | RegExp)[];
    };
    /**
     * Register apis globally
     *
     * @default false
     */
    global?: boolean;
    /**
     * Use `jsdom` or `happy-dom` to mock browser APIs
     *
     * @default false
     */
    dom?: boolean | 'jsdom' | 'happy-dom';
    /**
     * Update snapshot files
     *
     * @default false
     */
    update?: boolean;
    /**
     * Watch mode
     *
     * @default false
     */
    watch?: boolean;
    /**
     * Project root
     */
    root?: string;
    /**
     * Custom reporter for output
     */
    reporter?: Reporter;
}
interface CliOptions extends UserOptions {
    /**
     * Filters by name
     */
    cliFilters?: string[];
    /**
     * Path to the config file.
     *
     * Default resolving to one of:
     * - `vitest.config.js`
     * - `vitest.config.ts`
     * - `vite.config.js`
     * - `vite.config.ts`
     */
    config?: string | undefined;
}
interface ResolvedConfig extends Omit<Required<CliOptions>, 'config' | 'filters'> {
    config?: string;
    filters?: string[];
    depsInline: (string | RegExp)[];
    depsExternal: (string | RegExp)[];
    snapshotOptions: SnapshotStateOptions;
}
declare type RunMode = 'run' | 'skip' | 'only' | 'todo';
declare type TaskState = RunMode | 'pass' | 'fail';
declare type ComputeMode = 'serial' | 'concurrent';
interface TaskBase {
    id: string;
    name: string;
    mode: RunMode;
    computeMode: ComputeMode;
    suite?: Suite;
    file?: File;
    result?: TaskResult;
}
interface TaskResult {
    state: TaskState;
    start: number;
    end?: number;
    error?: unknown;
}
declare type TaskResultPack = [id: string, result: TaskResult | undefined];
interface Suite extends TaskBase {
    type: 'suite';
    tasks: Task[];
}
interface File extends Suite {
    filepath: string;
}
interface Test extends TaskBase {
    type: 'test';
    suite: Suite;
    result?: TaskResult;
}
declare type Task = Test | Suite | File;
declare type TestFunction = () => Awaitable<void>;
declare type TestCollectorFn = (name: string, fn: TestFunction, timeout?: number) => void;
interface ConcurrentCollector {
    (name: string, fn: TestFunction, timeout?: number): void;
    only: TestCollectorFn;
    skip: TestCollectorFn;
    todo: (name: string) => void;
}
interface OnlyCollector {
    (name: string, fn: TestFunction, timeout?: number): void;
    concurrent: TestCollectorFn;
}
interface SkipCollector {
    (name: string, fn: TestFunction, timeout?: number): void;
    concurrent: TestCollectorFn;
}
interface TodoCollector {
    (name: string): void;
    concurrent: (name: string) => void;
}
interface TestCollector {
    (name: string, fn: TestFunction, timeout?: number): void;
    concurrent: ConcurrentCollector;
    only: OnlyCollector;
    skip: SkipCollector;
    todo: TodoCollector;
}
declare type HookListener<T extends any[]> = (...args: T) => Awaitable<void>;
interface SuiteHooks {
    beforeAll: HookListener<[Suite]>[];
    afterAll: HookListener<[Suite]>[];
    beforeEach: HookListener<[Test, Suite]>[];
    afterEach: HookListener<[Test, Suite]>[];
}
interface SuiteCollector {
    readonly name: string;
    readonly mode: RunMode;
    type: 'collector';
    test: TestCollector;
    tasks: (Suite | Test | SuiteCollector)[];
    collect: (file?: File) => Promise<Suite>;
    clear: () => void;
    on: <T extends keyof SuiteHooks>(name: T, ...fn: SuiteHooks[T]) => void;
}
declare type TestFactory = (test: (name: string, fn: TestFunction) => void) => Awaitable<void>;
interface GlobalContext {
    tasks: (SuiteCollector | Test)[];
    currentSuite: SuiteCollector | null;
}
interface Reporter {
    onStart?: (files?: string[]) => Awaitable<void>;
    onFinished?: (files?: File[]) => Awaitable<void>;
    onTaskUpdate?: (pack: TaskResultPack) => Awaitable<void>;
    onWatcherStart?: () => Awaitable<void>;
    onWatcherRerun?: (files: string[], trigger: string) => Awaitable<void>;
}
interface ModuleCache {
    promise?: Promise<any>;
    exports?: any;
    transformResult?: TransformResult;
}
interface WorkerContext {
    port: MessagePort;
    config: ResolvedConfig;
    files: string[];
    invalidates?: string[];
}
interface RpcMap {
    workerReady: [[], void];
    fetch: [[id: string], TransformResult | null | undefined];
    onCollected: [[files: File[]], void];
    onFinished: [[], void];
    onTaskUpdate: [[pack: TaskResultPack], void];
    onWatcherStart: [[], void];
    onWatcherRerun: [[files: string[], trigger: string], void];
    snapshotSaved: [[snapshot: SnapshotResult], void];
}
declare type RpcCall = <T extends keyof RpcMap>(method: T, ...args: RpcMap[T][0]) => Promise<RpcMap[T][1]>;
declare type RpcSend = <T extends keyof RpcMap>(method: T, ...args: RpcMap[T][0]) => void;
declare type RpcPayload<T extends keyof RpcMap = keyof RpcMap> = {
    id: string;
    method: T;
    args: RpcMap[T][0];
};
interface VitestContext {
    config: ResolvedConfig;
    server: ViteDevServer;
    state: StateManager;
    snapshot: SnapshotManager;
    reporter: Reporter;
}

export { CliOptions as C, File as F, GlobalContext as G, HookListener as H, ModuleCache as M, ResolvedConfig as R, SuiteCollector as S, TestFactory as T, UserOptions as U, VitestContext as V, WorkerContext as W, TestFunction as a, SuiteHooks as b, RunMode as c, TaskState as d, ComputeMode as e, TaskBase as f, TaskResult as g, TaskResultPack as h, Suite as i, Test as j, Task as k, TestCollector as l, Reporter as m, RpcMap as n, RpcCall as o, RpcSend as p, RpcPayload as q };
